library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ram_test is
    Port ( clk : in std_logic;   
		   reset_n : in std_logic;
		   address_out: out std_logic_vector (15 downto 0);
           datain:  in std_logic_vector (15 downto 0);
           dataout_out: out std_logic_vector ( 15 downto 0);
           cs_n: out std_logic;
           we_n: out std_logic;
           oe_n: out std_logic
           
); 
end ram_test;

architecture behavioral of ram_test is


type mem_state_type is (write1,write2,write3,write4,read1,read2,read3,read4);
signal mem_state: mem_state_type;

signal temp: std_logic_vector (15 downto 0);
signal dataout: std_logic_vector (15 downto 0);
signal address: std_logic_vector (15 downto 0);



begin


address_out <= address;
dataout_out <= dataout;

process (clk,reset_n)
begin
	if reset_n = '0' then		
		 address <= x"0000";
		 dataout <= x"0000";
		 temp <= x"0000";
		 cs_n    <= '1';
		 we_n    <= '1';
		 oe_n    <= '1';
	     mem_state <= write1;
      elsif clk'event and clk = '1' then
	   case mem_state is
	    when write1 =>
	                 address <= x"0000";
		             dataout <= x"0000";
		             temp <= x"0000";
		             cs_n    <= '1';
		 			 we_n    <= '1';
		 			 oe_n    <= '1';
		             mem_state <= write2;
	              
		 
	     when write2 =>  
	            if address = x"FFFF" then
	                 address <= x"0000";
		             dataout <= x"0000";
		             temp <= x"0000";
		             cs_n    <= '1';
		 			 we_n    <= '1';
		 			 oe_n    <= '1';
		             mem_state <= read1;  
	              else
	                 address <= address;
		             dataout <= address;
		             temp <= x"0000";
		             cs_n    <= '0';
		 			 we_n    <= '1';
		 			 oe_n    <= '1';
		             mem_state <= write3;    
	  	         end if;
	      when write3 => --
	                 address <= address;
		             dataout <= address;
		             temp <= x"0000";
		             cs_n    <= '0';
		 			 we_n    <= '0';
		 			 oe_n    <= '1';
		             mem_state <= write4;         
	   	  when write4 => --
	                 address <= address + 1;
		             dataout <= address;
		             temp <= x"0000";
		             cs_n    <= '1';
		 			 we_n    <= '1';
		 			 oe_n    <= '1';
		             mem_state <= write2;  
       
	      when read1 => --
	                 address <= x"0000";
		             dataout <= x"0000";
		             temp <= x"0000";
		             cs_n    <= '1';
		 			 we_n    <= '1';
		 			 oe_n    <= '1';
		             mem_state <= read2;     
		
		  when read2 =>  
	            if address = x"FFFF" then
	                 address <= x"0000";
		             dataout <= x"0000";
		             temp <= datain;
		             cs_n    <= '1';
		 			 we_n    <= '1';
		 			 oe_n    <= '1';
		             mem_state <= write1;  
	              else
	                 address <= address;
		             dataout <= x"0000";
		             temp <= x"0000";
		             cs_n    <= '0';
		 			 we_n    <= '1';
		 			 oe_n    <= '1';
		             mem_state <= read3;    
	  	         end if;
	      when read3 => --
	                 address <= address;
		             dataout <= x"0000";
		             temp <= datain;
		             cs_n    <= '0';
		 			 we_n    <= '1';
		 			 oe_n    <= '0';
		             mem_state <= read4;         
	   	  when read4 => --
	                 address <= address + 1;
		             dataout <= x"0000";
		             temp <= datain;
		             cs_n    <= '1';
		 			 we_n    <= '1';
		 			 oe_n    <= '1';
		             mem_state <= read2;  
		  when others =>		
		            address <= x"0000";
		 			dataout <= x"0000";
		 			temp <= x"0000";
		 			cs_n    <= '1';
		 			we_n    <= '1';
		 			oe_n    <= '1';
	     			mem_state <= write1;
      end case;
   end if;
end process;

end behavioral;